--[[
    Marker Maker for After Effects
    (C) 2012 あかつきみさき（みくちぃP）

    このJob Pluginについて
    VOCALOID3 vsqx データをAdobe After Effectsで読み込めるファイルに出力します.
    出力可能な形式はAE上で直接実行する「ExtendScript」とAEスクリプトft-MarkerExporterで読み込める「XML」,XML→JSON変換した「JSON」の3種類です.
    VOCALOID3上で入力されたノートに基づく位置にマーカーがついています.

    ExtendScriptファイルに含まれる情報
    マーカーのコメント・Flashキューポイントには,ひとつは対応するノートの歌詞情報,もうひとつには母音情報を含んでいます.
    歌詞情報には,スライダー制御エフェクトで,音階が入力されています.
    母音情報には,スライダー制御エフェクトで,母音に対応した値が入力されています.

    XMLファイルに含まれる情報
    マーカーのコメント・Flashキューポイントには,ひとつは対応するノートの歌詞情報,もうひとつには発音記号情報を含んでいます.
    マーカーにはデュレーション・ノートナンバー・ベロシティ情報が含まれています.

    JSONファイルに含まれる情報
    一部JSON用にXMLを個別変換(最上位xml→json)したJSONファイルです. 基本的なデータはXMLと同じ内容です.

    インストール
    VOCALOID3上でパスを設定する必要があります.
    適当なディレクトリ(フォルダー)に置いて,VOCALOID3の「ジョブ→Jobプラグインを管理」から該当ディレクトリ(フォルダー)へパスを設定すれば完了です.
    アンインストールはパスの削除,直接削除してください.

    動作環境
    VOCALOID3, VOCALOID3 Job Pluginが動作する環境

    出力したExtendScriptファイルの動作環境
    Adobe After Effects CS4以上

    出力したXMLファイルの動作環境
    ft-MarkerExporter v1.0の対応しているバージョン

    使用方法
    一つのトラック内の一つ以上のノートを含むパートを選択し,「ジョブ→Jobプラグインを実行」から実行してください.
    ファイル名を入力し,ファイル形式を選択してOKを押すと,vsqxファイルと同じディレクトリ(フォルダー)内に「vsqxファイル名.vsqx_入力ファイル名.選択拡張子名」のJファイルが生成されます.
    デフォルトの場合は「vsqxファイル名.vsqx_パート名_VOCALOID3MarkerData.jsx」になっています.
    複数のパートの同時書き出しには対応していません.一度に複数のパートで使用したい場合はパートを結合してください.

    出力したExtendScriptファイルの使用方法
    After Effects上でヌルレイヤーを生成したいコンポジションを選択し,「ファイル→スクリプト→スクリプトファイルの実行」からスクリプトを実行してください.
    選択したコンポジション内に2つ「パート名 Lyric (BPM ※)」「パート名 Vowel (BPM ※)」というレイヤー名のヌルレイヤーが作成されます.※はBPM値

    出力したXMLファイルの使用方法
    ft-MarkerExporterのヘルプをご覧ください.

    出力したJSONファイルの使用方法
    ファイルの互換などにご利用ください.

    母音情報について
    「パート名 Vowel (BPM ※)」についたマーカーの母音情報は,以下の法則に基づいて付いています.
    清音,濁音,半濁音,小文字の"ぁ,ぃ,ぅ,ぇ,ぉ,ゃ,ゅ,ょ,ゎ","ヶ"は,それぞれ対応した母音により「a」「i」「u」「e」「o」
    小文字の"っ",長音符,踊り字は一つ前のノートの母音
    "ん"は,「n」
    複数で構成される文字(きゃ)などは促音から判定
    それ以外のものは,ノートに入力された文字がそのままマーカーのメモになります.
    スライダー制御エフェクトのキーフレームの値は以下の法則に基づいて記入されています.
    a = 1, i = 2, u = 3, e = 4, o = 5, n = 6, other = 0.

    バージョン情報
    2013/03/14 Ver 6.0.1.0 Update アラートが正しく表示されない問題の修正. 一部関数名の変更.
    2013/03/14 Ver 6.0.0.0 Update 出力するExtendScriptファイルの処理速度の大幅な改善・軽量化.
                                  ノートナンバーの出力. 内部処理の見直し.
                                  AE日本語・英語以外にも対応. AEでの対応バージョンの変更(CS4以上).
    2012/12/11 Ver 5.0.0.1 Update 出力するExtendScriptファイルの軽量化. 内部処理の見直し.
    2012/09/20 Ver 5.0.0.0 Update 出力するExtendScriptファイルのUndo処理の追加・無名関数化・ファイルの軽量化. 母音情報のキーフレームのタイプをHOLDに変更.
                                  母音情報をより正確に取得できるように変更. ExtendScript処理速度の改善. 内部処理の見直し.
    2012/07/27 Ver 4.2.0.0 Update 英語版After Effectsに対応. 内部処理の見直し.
    2012/07/20 Ver 4.1.0.0 Update カタカナ歌詞の対応. ExtendScript書き出し時のProgress barの追加,生成するヌルレイヤーのプロパティ変更.
    2012/07/19 Ver 4.0.1.0 Update 内部処理の見直し.
    2012/06/03 Ver 4.0.0.0 Update ft-MarkerExporterに準じたXMLファイルの書き出しに対応. JSONファイルの書き出しに対応. 処理の見直し. 出力するExtendScriptファイルの軽量化.
    2012/06/01 Ver 3.2.0.0 Update Flashキューポイントのプロパティをセットする機能の追加.
    2012/06/01 Ver 3.1.1.0 Update 歌詞が「ki」の時に母音情報が正しく入力されないバグの修正. 促音例外分岐の分岐追加. EULAの廃止. CS6の対応.
    2012/03/31 Ver 3.1.0.0 Update AE上でプロジェクトパネルの一番上以外でのコンポジションで実行したときにVowel Dataを読めない問題の修正.
    2012/03/26 Ver 3.0.0.0 Update ExtendScript処理速度の改善. 母音情報処理の変更. EULA処理の実装.
    2012/03/26 Ver 2.2.1.1 Update バグの修正.
    2012/03/14 Ver 2.2.1.0 Update pluginIDの修正, ExtendScript初回起動時のEULAの追加実験(2.2.1.0では未実装)
    2012/02/10 Ver 2.2.0.0 Update VOCALOID3発音記号の正式対応, 歌詞に発音記号が直接入力されていた時にExtendScriptが実行不可になる問題に対応.
    2012/01/18 Ver 2.1.1.0 Update ローマ字の正式対応.
    2012/01/14 Ver 2.1.0.0 Update AE CS5以上での動作の改善.
    2012/01/14 Ver 2.0.0.0 Update AEでの対応バージョンの変更(CS4以前の再対応).
    2011/12/26 Ver 1.2.0.0 Update 母音情報からスライダ制御の追加. AEでの対応バージョンの変更(CS4以前の切り捨て).
    2011/12/25 Ver 1.1.0.0 Update ExtendScriptに母音情報を含むマーカーのレイヤー生成機能の追加,ヘルプの追加
    2011/12/24 Ver 1.0.1.0 Update 書き出すExtendScript内のコメントの更新, AEのバージョン判定の追加
    2011/12/16 Ver 1.0.0.0 Release

    プラグイン配布先 YAMAHA VOCALOID STORE
    http://shop.vocaloidstore.com/index.php?gOo=goods_details.dwt&goodsid=5717

    プラグイン配布先記事「音のデータとAfter Effects ～VOCALOID3と連携編～」
    http://ae-users.com/jp/resources/2012/03/音のデータとAfter-Effects-～VOCALOID3と連携編～/

    Q & A
    Q1. パートを選択しているのにExtendScriptファイルが出力されません.
    A1. 選択されているパートにノートを含んでいるか確認してください.選択されているパートとはMUSICAL EDITORでアクティブになっているパートのことです.
    Q2. 出力したExtendScriptファイルの歌詞データ(マーカーのメモ)がおかしい.
    A2. 作成されたExtendScriptファイルの歌詞情報はVOCALOID3上で実際にノートに入力されているデータを参照しています.
        例えば,「カレーはおいしかった」という歌詞だと,一例として「かれえわおいしかた」といったように,VOCALOIDに正しく歌わせるために入力された歌詞になっています.
    Q3. 日本語以外の歌詞の母音情報が正しく入力されていない.
    A3. 母音情報は歌詞が日本語・ローマ字の時にのみ有効です.
]]

--プラグインマニフェスト関数
function manifest()
    myManifest = {
        name = "Marker Maker for After Effects",
        comment = "ノートの情報をAfter Effects対応のデータに出力します",
        author = "Misaki Akatsuki_102",
        pluginID = "{C77C1315-42B1-4e53-8959-8FF8241A2942}",
        pluginVersion = "6.0.1.0",
        apiVersion = "3.0.0.1"
    }
    return myManifest
end

-- VOCALOID3 Jobプラグインスクリプトのエントリポイント
function main(processParam, envParam)
-- 実行時に渡されたパラメータを取得
    local songPosTick = processParam.songPosTick
    local note = {}
    local noteEx = {}
    local noteExList = {}
    local noteCount
    local retCode
    local idx
    local musicalPart = {}
    local musicalSinger = {}
    local msg
    local tempo = {}
    local tempoList = {}
    local tempoNum
    local timeSig = {}
    local timeSigList = {}
    local timeSigNum
    local seqname = {} -- シーケンスネーム
    local partname -- パートネーム
    local lyric -- 歌詞
    local vowel -- 母音
    local key = {} -- スライダーエフェクト
    local bpm -- BPM
    local min -- 秒数

    local minusArray --配列マイナス用変数

-- Musicalパート情報を取得
    retCode, musicalPart, note = VSGetMusicalPart()
    partname = musicalPart.name
    
-- パートの中のノートを取得してノートイベント配列へ格納
    VSSeekToBeginNote()
    idx = 1
    retCode, noteEx = VSGetNextNoteEx()
    while (retCode == 1) do
        noteExList[idx] = noteEx
        retCode, noteEx = VSGetNextNoteEx()
        idx = idx + 1
    end

-- 選択したパートに入っているノートを読み込み
    noteCount = table.getn(noteExList)
-- ノート = 0のエラー処理
    if (noteCount == 0) then
        VSMessageBox("ノートを含むパートを選択してください.", 0)
        return 0
    end

-- 拍子の取得
    VSSeekToBeginTimeSig()
    idx = 1
    retCode, timeSig = VSGetNextTimeSig()
    while (retCode == 1) do
        timeSigList[idx] = timeSig
        retCode, timeSig = VSGetNextTimeSig()
        idx = idx + 1
    end
    timeSigNum = table.getn(timeSigList)

-- テンポの取得
    VSSeekToBeginTempo()
    idx = 1
    retCode, tempo = VSGetNextTempo()
    while (retCode == 1) do
        tempoList[idx] = tempo
        retCode, tempo = VSGetNextTempo()
        idx = idx + 1
    end
    tempoNum = table.getn(tempoList)
    bpm = tempoList[tempoNum].tempo

-- ファイルパス取得
    local sequencePath = VSGetSequencePath()
    musicalSinger = VSGetMusicalPartSinger()

-- パラメータ入力ダイアログのウィンドウタイトルを設定
    VSDlgSetDialogTitle("名前を付けて保存")
    local dlgStatus
    local field = {}
    field.name = "fname"
    field.caption = "ファイル名(既存のファイルは上書きされます)"
    field.initialVal = partname
    field.type = 3
    dlgStatus = VSDlgAddField(field)

    field.name       = "selectftype"
    field.caption    = "書き出し形式の選択"
    field.initialVal =
        "ExtendScript ファイル (*.jsx)" ..
        ",XML ファイル (*.xml)" ..
        ",JSON ファイル (*.json)"
--        ",すべてのファイル (*.*)"
    field.type = 4
    dlgStatus  = VSDlgAddField(field)

-- パラメータ入力ダイアログを表示
    dlgStatus = VSDlgDoModal()
    if  (dlgStatus ~= 1) then
    -- OKボタンが押されなかったら終了
        return 0
    end

    local filename
    local selectfilename
    dlgStatus, filename = VSDlgGetStringValue("fname")
    dlgStatus, selectfilename =  VSDlgGetStringValue("selectftype")
    
-- 拡張子判別
    local checkfile = filename .. ".jsx"
    
    if(selectfilename == "XML ファイル (*.xml)") then
        checkfile = filename .. ".xml"
    elseif(selectfilename == "JSON ファイル (*.json)") then
        checkfile = filename .. ".json"
    end

--[[
    local msg
    msg =
        "入力されたパラメータは,\n" ..
        " ファイル名 = [" .. filename ..
        "]\n ファイルの種類 = [" .. selectfilename ..
        " \n ファイル名(拡張子付き) = [" .. checkfile ..
        "]\nです."
    VSMessageBox(msg, 0)
]]

-- ファイル入力開始
    io.output(sequencePath .. "_" .. checkfile)

    -- ExtendScript出力
    if(selectfilename == "ExtendScript ファイル (*.jsx)") then
        io.write('/*\n')
        io.write('  VOCALOID3 Job Plugin ' .. manifest().name .. ' ' .. manifest().pluginVersion .. '\n')
        io.write('    (C) あかつきみさき(みくちぃP)\n')
        io.write('  \n')
        io.write('  このスクリプトについて\n')
        io.write('    VOCALOID3 Job Plugin 「' .. manifest().name .. '」で出力されたものです.\n')
        io.write('    VOCALOID3上で入力されたノートに基づく位置にマーカーがついたヌルレイヤーを作成します.\n')
        io.write('    マーカーのメモには,ひとつは対応するノートの歌詞情報,もうひとつには母音情報を含んでいます.\n')
        io.write('    詳しくは' .. manifest().name .. 'のヘルプをご覧ください.\n')
        io.write('  \n')
        io.write('  使用方法\n')
        io.write('    After Effects上でヌルレイヤーを生成したいコンポジションを選択し,「ファイル→スクリプト→スクリプトファイルの実行」からスクリプトを実行してください.\n')
        io.write('    選択したコンポジション内に2つ「パート名 Lyric (BPM ※)」「パート名 Vowel (BPM ※)」というレイヤー名のヌルレイヤーが作成されます.※はBPM値\n')
        io.write('  \n')
        io.write('  動作環境\n')
        io.write('    Adobe After Effects CS4以上\n')
        io.write('  \n')
        io.write('  プラグイン配布先 YAMAHA VOCALOID STORE\n')
        io.write('    http://shop.vocaloidstore.com/index.php?gOo=goods_details.dwt&goodsid=5717\n')
        -- io.write('  プラグイン配布先記事「音のデータとAfter Effects ～VOCALOID3と連携編～」\n')
        -- io.write('    http://ae-users.com/jp/resources/2012/03/音のデータとAfter-Effects-～VOCALOID3と連携編～/\n')
        io.write('*/\n')
        io.write('\n')

-- 無名関数開始
        io.write('(function(){\n')
        io.write('  var MM4AEData, MM4AEUserData, lyricLayer, noteNumEffect, vowelEffect, vowelLayer, durArray, getLocalizedText, _i, isCompActive, vowelkeyArray, lyricArray, markerObj, minArray, _noteCount, runAEVersionCheck, selComp, setMarker, vowelArray, writtenToInfoPanel;\n')
        io.write('\n')
        
        io.write('  MM4AEData = (function() {\n')
        io.write('    var canRunVersion, scriptName, scriptURLName, scriptURLVersion, scriptVersionNumber;\n')
        io.write('    scriptName = "'.. manifest().name ..'";\n')
        io.write('    scriptURLName = "MarkerMakerforAfterEffects";\n')
        io.write('    scriptVersionNumber = "' .. manifest().pluginVersion .. '";\n')
        io.write('    canRunVersion = 9.0;\n')
        io.write('    return {\n')
        io.write('      getScriptName: function() {\n')
        io.write('        return scriptName;\n')
        io.write('      },\n')
        io.write('      getScriptURLName: function() {\n')
        io.write('        return scriptURLName;\n')
        io.write('      },\n')
        io.write('      getScriptVersionNumber: function() {\n')
        io.write('        return scriptVersionNumber;\n')
        io.write('      },\n')
        io.write('      getScriptURLVersion: function() {\n')
        io.write('        return scriptURLVersion;\n')
        io.write('      },\n')
        io.write('      getCanRunVersion: function() {\n')
        io.write('        return canRunVersion;\n')
        io.write('      }\n')
        io.write('    };\n')
        io.write('  })();\n')
        io.write('\n')

        io.write('  MM4AEUserData = {\n')
        io.write('    myAlert: function(msg) {\n')
        io.write('      return alert(msg, MM4AEData.getScriptName());\n')
        io.write('    },\n')
        io.write('    partName : "' .. partname ..  '",\n')
        io.write('    effectName : "ADBE Effect Parade",\n')
        io.write('    sliderEffectName : "ADBE Slider Control",\n')
        io.write('    sliderPropertyName : "ADBE Slider Control-0001",\n')
        io.write('    markerName : "ADBE Marker",\n')
        io.write('    noteCount : ' .. noteCount .. ',\n')
        io.write('    bpm : ' .. bpm .. '\n')
        io.write('  };\n')
        io.write('\n')

        io.write('  AVLayer.prototype.setLayerMode = function(name){\n')
        io.write('    this.name = name;\n')
        io.write('    this.guideLayer = true;\n')
        io.write('    this.enabled = false;\n')
        io.write('  };\n')
        io.write('\n')

        io.write('  writtenToInfoPanel = function(finished) {\n')
        io.write('    writeLn(MM4AEData.getScriptName());\n')
        io.write('    finished ? writeLn("completed") : writeLn("incompleted");\n')
        io.write('    writeLn("");\n')
        io.write('  };\n')
        io.write('\n')

        io.write('  runAEVersionCheck = function(AEVersion) {\n')
        io.write('    if (parseFloat(app.version) < AEVersion) {\n')
        io.write('      MM4AEUserData.myAlert("This script requires After Effects " + (AEVersion.toFixed(1)) + " or greater.");\n')
        io.write('      writtenToInfoPanel(false);\n')
        io.write('      return false;\n')
        io.write('    } else {\n')
        io.write('      return true;\n')
        io.write('    }\n')
        io.write('  };\n')
        io.write('\n')

        io.write('  isCompActive = function(selComp) {\n')
        io.write('    if (!(selComp && selComp instanceof CompItem)) {\n')
        io.write('      MM4AEUserData.myAlert("Select Composition");\n')
        io.write('      writtenToInfoPanel(false);\n')
        io.write('      return false;\n')
        io.write('    } else {\n')
        io.write('      return true;\n')
        io.write('    }\n')
        io.write('  };\n')
        io.write('  \n')

--        io.write('  // レイヤーマーカー設定関数\n')
        io.write('  setMarker = function(selLayer, min, dur, commentMar, key) {\n')
        io.write('    markerObj = new MarkerValue(commentMar, "", "", "", commentMar, true);\n')
        io.write('    markerObj.duration = dur;\n')
        io.write('    selLayer.property(MM4AEUserData.markerName).setValueAtTime(min, markerObj);\n')
        io.write('  };\n')
        io.write('\n')
    
        io.write('  if (!runAEVersionCheck(MM4AEData.getCanRunVersion())) {\n')
        io.write('    return 0;\n')
        io.write('  }\n')
        io.write('  \n')

        io.write('  selComp = app.project.activeItem;\n')
        io.write('  \n')

        io.write('  if (!isCompActive(selComp)) {\n')
        io.write('    return 0;\n')
        io.write('  }\n')
        io.write('\n')

        io.write('  app.beginUndoGroup(MM4AEData.getScriptName());\n')
        
    -- レイヤーセッティング
        io.write('  lyricLayer = selComp.layers.addNull();\n')
        io.write('  vowelLayer = selComp.layers.addNull();\n')
        io.write('\n')

        io.write('  lyricLayer.setLayerMode(MM4AEUserData.partName + \" Lyric (BPM \" + MM4AEUserData.bpm + \")\");\n')
        io.write('  vowelLayer.setLayerMode(MM4AEUserData.partName + \" Vowel (BPM \" + MM4AEUserData.bpm + \")\");\n')
        io.write('\n')

        io.write('  lyricLayer.property(MM4AEUserData.effectName).addProperty(MM4AEUserData.sliderEffectName);\n')
        io.write('  vowelLayer.property(MM4AEUserData.effectName).addProperty(MM4AEUserData.sliderEffectName);\n')
        io.write('\n')

        io.write('  noteNumEffect = lyricLayer.effect(MM4AEUserData.sliderEffectName)(MM4AEUserData.sliderPropertyName);\n')
        io.write('  vowelEffect = vowelLayer.effect(MM4AEUserData.sliderEffectName)(MM4AEUserData.sliderPropertyName);\n')
        io.write('\n')

        io.write('  lyricLayer.effect(MM4AEUserData.sliderEffectName).name = "Note Number Data";\n')
        io.write('  vowelLayer.effect(MM4AEUserData.sliderEffectName).name = "Vowel Data";\n')
        io.write('\n')



    -- 時間データ挿入
        io.write('  minArray = [')
        for idx = 1, noteCount do
            setArrayLoop(idx, noteCount, tickToMin(noteExList[idx].posTick, bpm))
        end
    -- デュレーションデータ挿入
        io.write('  durArray = [')
        for idx = 1, noteCount do
            setArrayLoop(idx, noteCount, tickToMin(noteExList[idx].durTick, bpm))
        end
        io.write('  \n')
    
    -- 歌詞データ挿入
        io.write('  lyricArray = [')
        for idx = 1, noteCount do
            setArrayLoop(idx, noteCount, parseLyric(noteExList[idx].lyric))
        end
        
    -- 母音入力開始
        io.write('  vowelArray = [')
        for idx = 1, noteCount do
            lyric = noteExList[idx].lyric

    -- 母音判別 V3仕様変更対応の場合を考え,ローマ字は残す
    -- 促音例外分岐
            if (lyric == "っ") then
                minusArray = 0
                while (lyric == "-" or lyric == "ー" or lyric == "っ" or lyric == "xtu" or lyric == "ltu") do
                    minusArray = minusArray + 1
                    idx2 = idx - minusArray
                    lyric = noteExList[idx2].lyric
                end
            end
            if (lyric == "xtu") then
                minusArray = 0
                while (lyric == "-" or lyric == "ー" or lyric == "っ" or lyric == "xtu" or lyric == "ltu") do
                    minusArray = minusArray + 1
                    idx2 = idx - minusArray
                    lyric = noteExList[idx2].lyric
                end
            end
            if (lyric == "ltu") then
                minusArray = 0
                while (lyric == "-" or lyric == "ー" or lyric == "っ" or lyric == "xtu" or lyric == "ltu") do
                    minusArray = minusArray + 1
                    idx2 = idx - minusArray
                    lyric = noteExList[idx2].lyric
                end
            end
    -- 長音符例外分岐
            if (lyric == "ー") then
                minusArray = 0
                while (lyric == "-" or lyric == "ー" or lyric == "っ" or lyric == "xtu" or lyric == "ltu") do
                    minusArray = minusArray + 1
                    idx2 = idx - minusArray
                    lyric = noteExList[idx2].lyric
                end
            end
            if (lyric == "-") then
                minusArray = 0
                while (lyric == "-" or lyric == "ー" or lyric == "っ" or lyric == "xtu" or lyric == "ltu") do
                    minusArray = minusArray + 1
                    idx2 = idx - minusArray
                    lyric = noteExList[idx2].lyric
                end
            end

    -- 踊り字例外分岐
            if (lyric == "々") then
                idx2 = idx - 1
                lyric = noteExList[idx2].lyric
            end
            if (lyric == "ヽ") then
                idx2 = idx - 1
                lyric = noteExList[idx2].lyric
            end
            if (lyric == "ゝ") then
                idx2 = idx - 1
                lyric = noteExList[idx2].lyric
            end
            if (lyric == "ゞ") then
                idx2 = idx - 1
                lyric = noteExList[idx2].lyric
            end
            if (lyric == "ヾ") then
                idx2 = idx - 1
                lyric = noteExList[idx2].lyric
            end
            if (lyric == "〻") then
                idx2 = idx - 1
                lyric = noteExList[idx2].lyric
            end

            vowel = lyricToVowel(lyric)
            key[idx] = vowelToKey(vowel)

            setArrayLoop(idx, noteCount, lyricToVowel(lyric))
        end
        io.write('  \n')

        -- 音階データ挿入
        io.write('  noteNumArray = [')
        for idx = 1, noteCount do
            setArrayLoop(idx, noteCount, noteExList[idx].noteNum)
        end

        -- 母音キーフレーム用
        io.write('  vowelkeyArray = [')
        for idx = 1, noteCount do
            setArrayLoop(idx, noteCount, key[idx])
        end

        io.write('\n')

        io.write('  noteNumEffect.setValuesAtTimes(minArray, noteNumArray);\n')
        io.write('  vowelEffect.setValuesAtTimes(minArray, vowelkeyArray);\n')
        io.write('\n')

        io.write('  for(_i=0, _noteCount=MM4AEUserData.noteCount; _i<_noteCount; _i++){\n')
        io.write('    setMarker(lyricLayer, minArray[_i], durArray[_i], lyricArray[_i], vowelkeyArray[_i]);\n')
        io.write('    setMarker(vowelLayer, minArray[_i], durArray[_i], vowelArray[_i], vowelkeyArray[_i]);\n')
        io.write('    noteNumEffect.setInterpolationTypeAtKey(_i+1, KeyframeInterpolationType.HOLD, KeyframeInterpolationType.HOLD);\n')
        io.write('    vowelEffect.setInterpolationTypeAtKey(_i+1, KeyframeInterpolationType.HOLD, KeyframeInterpolationType.HOLD);\n')
        io.write('  }\n')

-- 440*2^(NoteNumber-69)/12
-- Math.pow(440*2, (effect("ch_1_pitch")("スライダー")-69)/12)

--[[
        io.write('  if(confirm(\"Add BPM Layers?\", false, MM4AEData.getScriptName())){\n')
        io.write('    var V3bpm11 = selComp.layers.addNull();\n')
        io.write('    V3bpm11.setLayerMode(\"BPM \" + MM4AEUserData.bpm + \" (1/1)\";\n'))
        io.write('    \n')
        io.write('    var V3bpm12 = V3bpm11.duplicate();\n')
        io.write('    V3bpm12.setLayerMode(\"BPM \" + MM4AEUserData.bpm + \" (1/2)\";\n'))
        io.write('    \n')
        io.write('    var V3bpm14 = V3bpm12.duplicate();\n')
        io.write('    V3bpm14.setLayerMode(\"BPM \" + MM4AEUserData.bpm + \" (1/4)\";\n'))
        io.write('    \n')
        io.write('    var V3bpm18 = V3bpm14.duplicate();\n')
        io.write('    V3bpm18.setLayerMode(\"BPM \" + MM4AEUserData.bpm + \" (1/8)\";\n'))
        io.write('  }\n')
        ]]
        io.write('\n')
        io.write('  writtenToInfoPanel(true);\n')
        io.write('\n')
        io.write('  app.endUndoGroup();\n')
        io.write('\n')
        io.write('}).call(this);\n')

        
    elseif(selectfilename == "XML ファイル (*.xml)") then
-- xml出力
        io.write('<xml>\n')
        io.write('  <general>\n')
        io.write('    <version>1</version>\n')
        io.write('  </general>\n')
        io.write('  <layers>\n')
        
        -- レイヤーの名前配列
        local layerName = {"Lyric", "phonemes"}
        local paramName = {"noteNum", "velocity", "Singer"}
        local event = "true"
        
        for layercount = 1, 2 do
            io.write('    <layer name="' .. partname .. ' ' .. layerName[layercount] .. ' (BPM \" + MM4AEUserData.bpm + \")" index="' .. layercount .. '">\n')
            for idx = 1, noteCount do
                if (layercount == 1)then
                    markervalue = noteExList[idx].lyric
                else
                    markervalue = noteExList[idx].phonemes
                end
                
                min = tickToMin(noteExList[idx].posTick, bpm)
                dur = tickToMin(noteExList[idx].durTick, bpm)
                noteNum = noteExList[idx].noteNum
                velocity = noteExList[idx].velocity
--                singerBS = musicalSinger.vBS
--                singerPC = musicalSinger.vPC
                
                io.write('      <marker>\n')
                io.write('        <comment value="' .. markervalue ..  '"/>\n')
                io.write('        <time value="' .. min ..  '"/>\n')
                io.write('        <duration value="' .. dur ..  '"/>\n')
                io.write('        <cuePointName value="' .. markervalue ..  '"/>\n')
                io.write('        <eventCuePoint value="' .. event ..  '"/>\n')
                io.write('        <chapter value=""/>\n')
                io.write('        <frameTarget value=""/>\n')
                io.write('        <url value=""/>\n')
                io.write('        <params>\n')
                io.write('          <param name="' .. paramName[1] .. '" value="' .. noteNum ..  '"/>\n')
                io.write('          <param name="' .. paramName[2] .. '" value="' .. velocity ..  '"/>\n')
                io.write('          <param name="' .. paramName[3] .. '" value="Singer"/>\n')
                io.write('        </params>\n')
                io.write('      </marker>\n')
            end
            
            io.write('    </layer>\n')
        end
        io.write('  </layers>\n')
        io.write('</xml>')

    else
    -- JSON
        -- レイヤーの名前配列
        local layerName = {"Lyric", "phonemes"}
        local paramName = {"noteNum", "velocity", "Singer"}
        local event = "true"
    
        io.write('{\n')
        io.write('  "json": {\n')
        io.write('    "general": { "version": "1" },\n')
        io.write('    "layers": {\n')
        io.write('      "layer": {\n')
        
        for layercount = 1, 2 do
            io.write('        "-name": "' .. partname .. ' ' .. layerName[layercount] .. ' (BPM \" + MM4AEUserData.bpm + \")",\n')
            io.write('        "-index": "' .. layercount .. '",\n')
            
            for idx = 1, noteCount do
                if (layercount == 1)then
                    markervalue = noteExList[idx].lyric
                else
                    markervalue = noteExList[idx].phonemes
                end
                min = tickToMin(noteExList[idx].posTick, bpm)
                dur = tickToMin(noteExList[idx].durTick, bpm)
                noteNum = noteExList[idx].noteNum
                velocity = noteExList[idx].velocity
--                singerBS = musicalSinger.vBS
--                singerPC = musicalSinger.vPC
                
                io.write('        "marker": {\n')
                io.write('          "comment": { "-value": "' .. markervalue ..  '" },\n')
                io.write('          "time": { "-value": "' .. min ..  '" },\n')
                io.write('          "duration": { "-value": "' .. dur ..  '" },\n')
                io.write('          "cuePointName": { "-value": "' .. markervalue ..  '" },\n')
                io.write('          "eventCuePoint": { "-value": "' .. event ..  '" },\n')
                io.write('          "chapter": {\n')
                io.write('            \n')
                io.write('          },\n')
                io.write('          "frameTarget": {\n')
                io.write('            \n')
                io.write('          },\n')
                io.write('          "url": {\n')
                io.write('            \n')
                io.write('          },\n')
                io.write('          "params": {\n')
                io.write('            "param": [\n')
                io.write('              {\n')
                io.write('                "-name": "' .. paramName[1] .. '",\n')
                io.write('                "-value": "' .. noteNum ..  '"\n')
                io.write('              },\n')
                io.write('            "param": [\n')
                io.write('              {\n')
                io.write('                "-name": "' .. paramName[2] .. '",\n')
                io.write('                "-value": "' .. velocity ..  '"\n')
                io.write('              },\n')
                io.write('            "param": [\n')
                io.write('              {\n')
                io.write('                "-name": "' .. paramName[3] .. '",\n')
                io.write('                "-value": ""\n')
                io.write('              },\n')
                io.write('            ]\n')
                io.write('          }\n')
                io.write('        }\n')
            end
            io.write('      }\n')
        end
        io.write('    }\n')
        io.write('  }\n')
        io.write('}\n')
    end

-- 正常終了.
    return 0
end






-- tickをミリ秒に変換する
function tickToMin(tick, bpm)
    return tick / 480 * 60 / bpm
end

function setExtendScriptArray(arrayName, noteCount, value)
    io.write('  ' .. arrayName .. ' = [')
    for idx = 1, noteCount do
        if(idx == noteCount) then
            io.write('\"' .. value .. '\"];\n')
        else
            io.write('\"' .. value .. '\",')
        end
    end
end


function setArrayLoop(idx, noteCount, value)
    if(idx == noteCount) then
        io.write('\"' .. value .. '\"];\n')
    else
        io.write('\"' .. value .. '\",')
    end
end


--特殊歌詞変換関数
function parseLyric(lyric)
-- 発音記号エラー回避例外分岐
    if (lyric == 'h\\') then
        lyric = "h"
    elseif (lyric == 'p\\') then
        lyric = "h"
    elseif (lyric == "p\\'") then
        lyric = "f"
    elseif (lyric == 'N\\') then
        lyric = "n"
-- 英語
    elseif (lyric == '@') then
        lyric = "a"
    elseif (lyric == 'i:') then
        lyric = "i"
    elseif (lyric == '{') then
        lyric = "a"
    elseif (lyric == 'O:') then
        lyric = "a"
    elseif (lyric == 'u:') then
        lyric = "o"
    elseif (lyric == '@r') then
        lyric = "a"
    elseif (lyric == '@U') then
        lyric = "o"
    elseif (lyric == 'l@') then
        lyric = "i"
    elseif (lyric == 'e@') then
        lyric = "e"
    elseif (lyric == 'U@') then
        lyric = "o"
    elseif (lyric == 'O@') then
        lyric = "u"
    elseif (lyric == 'Q@') then
        lyric = "a"
    end
    return lyric
end

-- 歌詞-母音変換関数
function lyricToVowel(lyric)
    local vowel
-- 発音記号分岐
    if (lyric == "a") then
        vowel = "a"
    elseif (lyric == "i") then
        vowel = "i"
    elseif (lyric == "M") then
        vowel = "u"
    elseif (lyric == "e") then
        vowel = "e"
    elseif (lyric == "o") then
        vowel = "o"
    elseif (lyric == "k") then
        vowel = lyric
    elseif (lyric == "k'") then
        vowel = lyric
    elseif (lyric == "g") then
        vowel = lyric
    elseif (lyric == "g'") then
        vowel = lyric
    elseif (lyric == "N") then
        vowel = "g"
    elseif (lyric == "N'") then
        vowel = "g"
    elseif (lyric == "s") then
        vowel = lyric
    elseif (lyric == "S") then
        vowel = "s"
    elseif (lyric == "z") then
        vowel = lyric
    elseif (lyric == "Z") then
        vowel = "j"
    elseif (lyric == "dz") then
        vowel = "z"
    elseif (lyric == "dZ") then
        vowel = "j"
    elseif (lyric == "t") then
        vowel = lyric
    elseif (lyric == "t'") then
        vowel = "t"
    elseif (lyric == "ts") then
        vowel = "t"
    elseif (lyric == "tS") then
        vowel = "t"
    elseif (lyric == "d") then
        vowel = lyric
    elseif (lyric == "d'") then
        vowel = "d"
    elseif (lyric == "n") then
        vowel = lyric
    elseif (lyric == "J") then
        vowel = "n"
    elseif (lyric == "h") then
        vowel = lyric
    elseif (lyric == 'h\\') then
        vowel = "h"
    elseif (lyric == "C") then
        vowel = "h"
    elseif (lyric == 'p\\') then
        vowel = "h"
    elseif (lyric == "p\\'") then
        vowel = "f"
    elseif (lyric == "b") then
        vowel = lyric
    elseif (lyric == "b'") then
        vowel = "b"
    elseif (lyric == "p") then
        vowel = lyric
    elseif (lyric == "p'") then
        vowel = "p"
    elseif (lyric == "m") then
        vowel = lyric
    elseif (lyric == "m'") then
        vowel = "m"
    elseif (lyric == "j") then
        vowel = "y"
    elseif (lyric == "4") then
        vowel = "r"
    elseif (lyric == "4'") then
        vowel = "r"
    elseif (lyric == "w") then
        vowel = lyric
    elseif (lyric == "N\\") then
        vowel = "n"

-- 清音
    elseif (lyric == "あ" or lyric == "ア") then
        vowel = "a"
    elseif (lyric == "い" or lyric == "イ") then
        vowel = "i"
    elseif (lyric == "う" or lyric == "ウ") then
        vowel = "u"
    elseif (lyric == "え" or lyric == "エ") then
        vowel = "e"
    elseif (lyric == "お" or lyric == "オ") then
        vowel = "o"
    elseif (lyric == "か" or lyric == "カ") then
        vowel = "a"
    elseif (lyric == "き" or lyric == "キ") then
        vowel = "i"
    elseif (lyric == "く" or lyric == "ク") then
        vowel = "u"
    elseif (lyric == "け" or lyric == "ケ") then
        vowel = "e"
    elseif (lyric == "こ" or lyric == "コ") then
        vowel = "o"
    elseif (lyric == "さ" or lyric == "サ") then
        vowel = "a"
    elseif (lyric == "し" or lyric == "シ") then
        vowel = "i"
    elseif (lyric == "す" or lyric == "ス") then
        vowel = "u"
    elseif (lyric == "せ" or lyric == "セ") then
        vowel = "e"
    elseif (lyric == "そ" or lyric == "ソ") then
        vowel = "o"
    elseif (lyric == "た" or lyric == "タ") then
        vowel = "a"
    elseif (lyric == "ち" or lyric == "チ") then
        vowel = "i"
    elseif (lyric == "つ" or lyric == "ツ") then
        vowel = "u"
    elseif (lyric == "て" or lyric == "テ") then
        vowel = "e"
    elseif (lyric == "と" or lyric == "ト") then
        vowel = "o"
    elseif (lyric == "な" or lyric == "ナ") then
        vowel = "a"
    elseif (lyric == "に" or lyric == "ニ") then
        vowel = "i"
    elseif (lyric == "ぬ" or lyric == "ヌ") then
        vowel = "u"
    elseif (lyric == "ね" or lyric == "ネ") then
        vowel = "e"
    elseif (lyric == "の" or lyric == "ノ") then
        vowel = "o"
    elseif (lyric == "は" or lyric == "ハ") then
        vowel = "a"
    elseif (lyric == "ひ" or lyric == "ヒ") then
        vowel = "i"
    elseif (lyric == "ふ" or lyric == "フ") then
        vowel = "u"
    elseif (lyric == "へ" or lyric == "ヘ") then
        vowel = "e"
    elseif (lyric == "ほ" or lyric == "ホ") then
        vowel = "o"
    elseif (lyric == "ま" or lyric == "マ") then
        vowel = "a"
    elseif (lyric == "み" or lyric == "ミ") then
        vowel = "i"
    elseif (lyric == "む" or lyric == "ム") then
        vowel = "u"
    elseif (lyric == "め" or lyric == "メ") then
        vowel = "e"
    elseif (lyric == "も" or lyric == "モ") then
        vowel = "o"
    elseif (lyric == "や" or lyric == "ヤ") then
        vowel = "a"
    elseif (lyric == "ゐ" or lyric == "ヰ") then
        vowel = "i"
    elseif (lyric == "ゆ" or lyric == "ユ") then
        vowel = "u"
    elseif (lyric == "ゑ" or lyric == "ヱ") then
        vowel = "e"
    elseif (lyric == "よ" or lyric == "ヨ") then
        vowel = "o"
    elseif (lyric == "ら" or lyric == "ラ") then
        vowel = "a"
    elseif (lyric == "り" or lyric == "リ") then
        vowel = "i"
    elseif (lyric == "る" or lyric == "ル") then
        vowel = "u"
    elseif (lyric == "れ" or lyric == "レ") then
        vowel = "e"
    elseif (lyric == "ろ" or lyric == "ロ") then
        vowel = "o"
    elseif (lyric == "わ" or lyric == "ワ") then
        vowel = "a"
    elseif (lyric == "を" or lyric == "ヲ") then
        vowel = "o"
    elseif (lyric == "ん" or lyric == "ン") then
        vowel = "n"
    elseif (lyric == "ヶ") then
        vowel = "a"
-- 濁音
    elseif (lyric == "が" or lyric == "ガ") then
        vowel = "a"
    elseif (lyric == "ぎ" or lyric == "ギ") then
        vowel = "i"
    elseif (lyric == "ぐ" or lyric == "グ") then
        vowel = "u"
    elseif (lyric == "げ" or lyric == "ゲ") then
        vowel = "e"
    elseif (lyric == "ご" or lyric == "ゴ") then
        vowel = "o"
    elseif (lyric == "ざ" or lyric == "ザ") then
        vowel = "a"
    elseif (lyric == "じ" or lyric == "ジ") then
        vowel = "i"
    elseif (lyric == "ず" or lyric == "ズ") then
        vowel = "u"
    elseif (lyric == "ぜ" or lyric == "ゼ") then
        vowel = "e"
    elseif (lyric == "ぞ" or lyric == "ゾ") then
        vowel = "o"
    elseif (lyric == "だ" or lyric == "ダ") then
        vowel = "a"
    elseif (lyric == "ぢ" or lyric == "ヂ") then
        vowel = "i"
    elseif (lyric == "づ" or lyric == "ヅ") then
        vowel = "u"
    elseif (lyric == "で" or lyric == "デ") then
        vowel = "e"
    elseif (lyric == "ど" or lyric == "ド") then
        vowel = "o"
    elseif (lyric == "ば" or lyric == "バ") then
        vowel = "a"
    elseif (lyric == "び" or lyric == "ビ") then
        vowel = "i"
    elseif (lyric == "ぶ" or lyric == "ブ") then
        vowel = "u"
    elseif (lyric == "べ" or lyric == "ベ") then
        vowel = "e"
    elseif (lyric == "ぼ" or lyric == "ボ") then
        vowel = "o"
    elseif (lyric == "ヴ") then
        vowel = "u"
-- 半濁音
    elseif (lyric == "ぱ" or lyric == "パ") then
        vowel = "a"
    elseif (lyric == "ぴ" or lyric == "ピ") then
        vowel = "i"
    elseif (lyric == "ぷ" or lyric == "プ") then
        vowel = "u"
    elseif (lyric == "ぺ" or lyric == "ペ") then
        vowel = "e"
    elseif (lyric == "ぽ" or lyric == "ポ") then
        vowel = "o"
-- 小文字
    elseif (lyric == "ぁ" or lyric == "ァ") then
        vowel = "a"
    elseif (lyric == "ぃ" or lyric == "ィ") then
        vowel = "i"
    elseif (lyric == "ぅ" or lyric == "ゥ") then
        vowel = "u"
    elseif (lyric == "ぇ" or lyric == "ェ") then
        vowel = "e"
    elseif (lyric == "ぉ" or lyric == "ォ") then
        vowel = "o"
    elseif (lyric == "ゃ" or lyric == "ャ") then
        vowel = "a"
    elseif (lyric == "ゅ" or lyric == "ュ") then
        vowel = "u"
    elseif (lyric == "ょ" or lyric == "ョ") then
        vowel = "o"
    elseif (lyric == "ゎ" or lyric == "ヮ") then
        vowel = "a"
-- 複数文字(清音)
    elseif (lyric == "きゃ" or lyric == "キャ") then
        vowel = "a"
    elseif (lyric == "きぃ" or lyric == "キィ") then
        vowel = "i"
    elseif (lyric == "きゅ" or lyric == "キュ") then
        vowel = "u"
    elseif (lyric == "きぇ" or lyric == "キェ") then
        vowel = "e"
    elseif (lyric == "きょ" or lyric == "キョ") then
        vowel = "o"
    elseif (lyric == "くぁ" or lyric == "クァ") then
        vowel = "a"
    elseif (lyric == "くぃ" or lyric == "クィ") then
        vowel = "i"
    elseif (lyric == "くゅ" or lyric == "クゥ") then
        vowel = "u"
    elseif (lyric == "くぇ" or lyric == "クェ") then
        vowel = "e"
    elseif (lyric == "くぉ" or lyric == "クォ") then
        vowel = "o"
    elseif (lyric == "くゃ" or lyric == "クャ") then
        vowel = "a"
    elseif (lyric == "くゅ" or lyric == "クュ") then
        vowel = "u"
    elseif (lyric == "くょ" or lyric == "クョ") then
        vowel = "o"
    elseif (lyric == "しゃ" or lyric == "シャ") then
        vowel = "a"
    elseif (lyric == "しぃ" or lyric == "シィ") then
        vowel = "i"
    elseif (lyric == "しゅ" or lyric == "シュ") then
        vowel = "u"
    elseif (lyric == "しぇ" or lyric == "シェ") then
        vowel = "e"
    elseif (lyric == "しょ" or lyric == "ショ") then
        vowel = "o"
    elseif (lyric == "ちゃ" or lyric == "チャ") then
        vowel = "a"
    elseif (lyric == "ちぃ" or lyric == "チィ") then
        vowel = "i"
    elseif (lyric == "ちゅ" or lyric == "チュ") then
        vowel = "u"
    elseif (lyric == "ちぇ" or lyric == "チェ") then
        vowel = "e"
    elseif (lyric == "ちょ" or lyric == "チョ") then
        vowel = "o"
    elseif (lyric == "てゃ" or lyric == "テャ") then
        vowel = "a"
    elseif (lyric == "てぃ" or lyric == "ティ") then
        vowel = "i"
    elseif (lyric == "てゅ" or lyric == "テュ") then
        vowel = "u"
    elseif (lyric == "てぇ" or lyric == "テェ") then
        vowel = "e"
    elseif (lyric == "てょ" or lyric == "テョ") then
        vowel = "o"
    elseif (lyric == "にゃ" or lyric == "ニャ") then
        vowel = "a"
    elseif (lyric == "にぃ" or lyric == "ニィ") then
        vowel = "i"
    elseif (lyric == "にゅ" or lyric == "ニョ") then
        vowel = "u"
    elseif (lyric == "にぇ" or lyric == "ニェ") then
        vowel = "e"
    elseif (lyric == "にょ" or lyric == "ニョ") then
        vowel = "o"
    elseif (lyric == "ひゃ" or lyric == "ヒャ") then
        vowel = "a"
    elseif (lyric == "ひぃ" or lyric == "ヒィ") then
        vowel = "i"
    elseif (lyric == "ひゅ" or lyric == "ヒュ") then
        vowel = "u"
    elseif (lyric == "ひぇ" or lyric == "ヒェ") then
        vowel = "e"
    elseif (lyric == "ひょ" or lyric == "ヒョ") then
        vowel = "o"
    elseif (lyric == "みゃ" or lyric == "ミャ") then
        vowel = "a"
    elseif (lyric == "みぃ" or lyric == "ミィ") then
        vowel = "i"
    elseif (lyric == "みゅ" or lyric == "ミュ") then
        vowel = "u"
    elseif (lyric == "みぇ" or lyric == "ミェ") then
        vowel = "e"
    elseif (lyric == "みょ" or lyric == "ミョ") then
        vowel = "o"
    elseif (lyric == "りゃ" or lyric == "リャ") then
        vowel = "a"
    elseif (lyric == "りぃ" or lyric == "リィ") then
        vowel = "i"
    elseif (lyric == "りゅ" or lyric == "リュ") then
        vowel = "u"
    elseif (lyric == "りぇ" or lyric == "リェ") then
        vowel = "e"
    elseif (lyric == "りょ" or lyric == "リョ") then
        vowel = "o"
    elseif (lyric == "うぁ" or lyric == "ウァ") then
        vowel = "a"
    elseif (lyric == "うぃ" or lyric == "ウィ") then
        vowel = "i"
    elseif (lyric == "うぅ" or lyric == "ウゥ") then
        vowel = "u"
    elseif (lyric == "うぇ" or lyric == "ウェ") then
        vowel = "e"
    elseif (lyric == "うぉ" or lyric == "ウォ") then
        vowel = "o"
-- 複数文字(濁音)
    elseif (lyric == "ヴぁ" or lyric == "ヴァ") then
        vowel = "a"
    elseif (lyric == "ヴぃ" or lyric == "ヴィ") then
        vowel = "i"
    elseif (lyric == "ヴぅ" or lyric == "ヴゥ") then
        vowel = "u"
    elseif (lyric == "ヴぇ" or lyric == "ヴェ") then
        vowel = "e"
    elseif (lyric == "ヴぉ" or lyric == "ヴォ") then
        vowel = "o"
    elseif (lyric == "ぎゃ" or lyric == "ギャ") then
        vowel = "a"
    elseif (lyric == "ぎぃ" or lyric == "ギィ") then
        vowel = "i"
    elseif (lyric == "ぎゅ" or lyric == "ギュ") then
        vowel = "u"
    elseif (lyric == "ぎぇ" or lyric == "ギェ") then
        vowel = "e"
    elseif (lyric == "ぎょ" or lyric == "ギョ") then
        vowel = "o"
    elseif (lyric == "ぐぁ" or lyric == "グァ") then
        vowel = "a"
    elseif (lyric == "ぐぃ" or lyric == "グィ") then
        vowel = "i"
    elseif (lyric == "ぐぅ" or lyric == "グゥ") then
        vowel = "u"
    elseif (lyric == "ぐぇ" or lyric == "グェ") then
        vowel = "e"
    elseif (lyric == "ぐぉ" or lyric == "グォ") then
        vowel = "o"
    elseif (lyric == "ぐゃ" or lyric == "グャ") then
        vowel = "a"
    elseif (lyric == "ぐゅ" or lyric == "グュ") then
        vowel = "u"
    elseif (lyric == "ぐょ" or lyric == "グォ") then
        vowel = "o"
    elseif (lyric == "じゃ" or lyric == "ジャ") then
        vowel = "a"
    elseif (lyric == "じぃ" or lyric == "ジィ") then
        vowel = "i"
    elseif (lyric == "じゅ" or lyric == "ジュ") then
        vowel = "u"
    elseif (lyric == "じぇ" or lyric == "ジェ") then
        vowel = "e"
    elseif (lyric == "じょ" or lyric == "ジョ") then
        vowel = "o"
    elseif (lyric == "ぢゃ" or lyric == "ヂァ") then
        vowel = "a"
    elseif (lyric == "ぢぃ" or lyric == "ヂィ") then
        vowel = "i"
    elseif (lyric == "ぢゅ" or lyric == "ヂゥ") then
        vowel = "u"
    elseif (lyric == "ぢぇ" or lyric == "ヂェ") then
        vowel = "e"
    elseif (lyric == "ぢょ" or lyric == "ヂォ") then
        vowel = "o"
    elseif (lyric == "でゃ" or lyric == "デャ") then
        vowel = "a"
    elseif (lyric == "でぃ" or lyric == "ディ") then
        vowel = "i"
    elseif (lyric == "でゅ" or lyric == "デュ") then
        vowel = "u"
    elseif (lyric == "でぇ" or lyric == "デェ") then
        vowel = "e"
    elseif (lyric == "でょ" or lyric == "デョ") then
        vowel = "o"
    elseif (lyric == "びゃ" or lyric == "ビャ") then
        vowel = "a"
    elseif (lyric == "びぃ" or lyric == "ビィ") then
        vowel = "i"
    elseif (lyric == "びゅ" or lyric == "ビュ") then
        vowel = "u"
    elseif (lyric == "びぇ" or lyric == "ビェ") then
        vowel = "e"
    elseif (lyric == "びょ" or lyric == "ビョ") then
        vowel = "o"
-- 複数文字(半濁音)
    elseif (lyric == "ぴゃ" or lyric == "ピャ") then
        vowel = "a"
    elseif (lyric == "ぴぃ" or lyric == "ピィ") then
        vowel = "i"
    elseif (lyric == "ぴゅ" or lyric == "ピュ") then
        vowel = "u"
    elseif (lyric == "ぴぇ" or lyric == "ピェ") then
        vowel = "e"
    elseif (lyric == "ぴょ" or lyric == "ピョ") then
        vowel = "o"
-- 2バイト英字処理(清書)
    elseif (lyric == "ａ") then
        vowel = "a"
    elseif (lyric == "ｉ") then
        vowel = "i"
    elseif (lyric == "ｕ") then
        vowel = "u"
    elseif (lyric == "ｅ") then
        vowel = "e"
    elseif (lyric == "ｏ") then
        vowel = "o"
    elseif (lyric == "ｎ") then
        vowel = "n"
-- ローマ字処理(清書)
    elseif (lyric == "a") then
        vowel = "a"
    elseif (lyric == "i") then
        vowel = "i"
    elseif (lyric == "u") then
        vowel = "u"
    elseif (lyric == "e") then
        vowel = "e"
    elseif (lyric == "o") then
        vowel = "o"
    elseif (lyric == "ka") then
        vowel = "a"
    elseif (lyric == "ki") then
        vowel = "i"
    elseif (lyric == "ku") then
        vowel = "u"
    elseif (lyric == "ke") then
        vowel = "e"
    elseif (lyric == "ko") then
        vowel = "o"
    elseif (lyric == "sa") then
        vowel = "a"
    elseif (lyric == "si") then
        vowel = "i"
    elseif (lyric == "shi") then
        vowel = "i"
    elseif (lyric == "su") then
        vowel = "u"
    elseif (lyric == "se") then
        vowel = "e"
    elseif (lyric == "so") then
        vowel = "o"
    elseif (lyric == "ta") then
        vowel = "a"
    elseif (lyric == "ti") then
        vowel = "i"
    elseif (lyric == "chi") then
        vowel = "i"
    elseif (lyric == "tu") then
        vowel = "u"
    elseif (lyric == "te") then
        vowel = "e"
    elseif (lyric == "to") then
        vowel = "o"
    elseif (lyric == "na") then
        vowel = "a"
    elseif (lyric == "ni") then
        vowel = "i"
    elseif (lyric == "nu") then
        vowel = "u"
    elseif (lyric == "ne") then
        vowel = "e"
    elseif (lyric == "no") then
        vowel = "o"
    elseif (lyric == "ha") then
        vowel = "a"
    elseif (lyric == "hi") then
        vowel = "i"
    elseif (lyric == "hu") then
        vowel = "u"
    elseif (lyric == "fu") then
        vowel = "u"
    elseif (lyric == "he") then
        vowel = "e"
    elseif (lyric == "ho") then
        vowel = "o"
    elseif (lyric == "ma") then
        vowel = "a"
    elseif (lyric == "mi") then
        vowel = "i"
    elseif (lyric == "mu") then
        vowel = "u"
    elseif (lyric == "me") then
        vowel = "e"
    elseif (lyric == "mo") then
        vowel = "o"
    elseif (lyric == "ya") then
        vowel = "a"
    elseif (lyric == "yi") then
        vowel = "i"
    elseif (lyric == "yu") then
        vowel = "u"
    elseif (lyric == "ye") then
        vowel = "e"
    elseif (lyric == "yo") then
        vowel = "o"
    elseif (lyric == "ra") then
        vowel = "a"
    elseif (lyric == "ri") then
        vowel = "i"
    elseif (lyric == "ru") then
        vowel = "u"
    elseif (lyric == "re") then
        vowel = "e"
    elseif (lyric == "ro") then
        vowel = "o"
    elseif (lyric == "wa") then
        vowel = "a"
    elseif (lyric == "wo") then
        vowel = "o"
    elseif (lyric == "n") then
        vowel = "n"
    elseif (lyric == "nn") then
        vowel = "n"
-- ローマ字処理(濁音)
    elseif (lyric == "ga") then
        vowel = "a"
    elseif (lyric == "gi") then
        vowel = "i"
    elseif (lyric == "gu") then
        vowel = "u"
    elseif (lyric == "ge") then
        vowel = "e"
    elseif (lyric == "go") then
        vowel = "o"
    elseif (lyric == "za") then
        vowel = "a"
    elseif (lyric == "zi") then
        vowel = "i"
    elseif (lyric == "ji") then
        vowel = "i"
    elseif (lyric == "zu") then
        vowel = "u"
    elseif (lyric == "ze") then
        vowel = "e"
    elseif (lyric == "zo") then
        vowel = "o"
    elseif (lyric == "da") then
        vowel = "a"
    elseif (lyric == "di") then
        vowel = "i"
    elseif (lyric == "du") then
        vowel = "u"
    elseif (lyric == "de") then
        vowel = "e"
    elseif (lyric == "do") then
        vowel = "o"
    elseif (lyric == "ba") then
        vowel = "a"
    elseif (lyric == "bi") then
        vowel = "i"
    elseif (lyric == "bu") then
        vowel = "u"
    elseif (lyric == "be") then
        vowel = "e"
    elseif (lyric == "bo") then
        vowel = "o"
-- ローマ字処理(半濁音)
    elseif (lyric == "pa") then
        vowel = "a"
    elseif (lyric == "pi") then
        vowel = "i"
    elseif (lyric == "pu") then
        vowel = "u"
    elseif (lyric == "pe") then
        vowel = "e"
    elseif (lyric == "po") then
        vowel = "o"
-- ローマ字処理(小文字)
    elseif (lyric == "xa") then
        vowel = "a"
    elseif (lyric == "xi") then
        vowel = "i"
    elseif (lyric == "xu") then
        vowel = "u"
    elseif (lyric == "xe") then
        vowel = "e"
    elseif (lyric == "xo") then
        vowel = "o"
    elseif (lyric == "xya") then
        vowel = "a"
    elseif (lyric == "xyu") then
        vowel = "u"
    elseif (lyric == "xyo") then
        vowel = "o"
    elseif (lyric == "xwa") then
        vowel = "a"
    elseif (lyric == "la") then
        vowel = "a"
    elseif (lyric == "li") then
        vowel = "i"
    elseif (lyric == "lu") then
        vowel = "u"
    elseif (lyric == "le") then
        vowel = "e"
    elseif (lyric == "lo") then
        vowel = "o"
    elseif (lyric == "lya") then
        vowel = "a"
    elseif (lyric == "lyu") then
        vowel = "u"
    elseif (lyric == "lyo") then
        vowel = "o"
    elseif (lyric == "lwa") then
        vowel = "a"
-- ローマ字処理(複数文字(清音))
    elseif (lyric == "kya") then
        vowel = "a"
    elseif (lyric == "kyi") then
        vowel = "i"
    elseif (lyric == "kyu") then
        vowel = "u"
    elseif (lyric == "kye") then
        vowel = "e"
    elseif (lyric == "kyo") then
        vowel = "o"
    elseif (lyric == "qa") then
        vowel = "a"
    elseif (lyric == "qi") then
        vowel = "i"
    elseif (lyric == "qu") then
        vowel = "u"
    elseif (lyric == "qe") then
        vowel = "e"
    elseif (lyric == "qo") then
        vowel = "o"
    elseif (lyric == "sha") then
        vowel = "a"
    elseif (lyric == "shu") then
        vowel = "u"
    elseif (lyric == "she") then
        vowel = "e"
    elseif (lyric == "sho") then
        vowel = "o"
    elseif (lyric == "sya") then
        vowel = "a"
    elseif (lyric == "syi") then
        vowel = "i"
    elseif (lyric == "syu") then
        vowel = "u"
    elseif (lyric == "sye") then
        vowel = "e"
    elseif (lyric == "syo") then
        vowel = "o"
    elseif (lyric == "cha") then
        vowel = "a"
    elseif (lyric == "chu") then
        vowel = "u"
    elseif (lyric == "che") then
        vowel = "e"
    elseif (lyric == "cho") then
        vowel = "o"
    elseif (lyric == "tya") then
        vowel = "a"
    elseif (lyric == "tyi") then
        vowel = "i"
    elseif (lyric == "tyu") then
        vowel = "u"
    elseif (lyric == "tye") then
        vowel = "e"
    elseif (lyric == "tyo") then
        vowel = "o"
    elseif (lyric == "tha") then
        vowel = "a"
    elseif (lyric == "thi") then
        vowel = "i"
    elseif (lyric == "thu") then
        vowel = "u"
    elseif (lyric == "the") then
        vowel = "e"
    elseif (lyric == "tho") then
        vowel = "o"
    elseif (lyric == "nya") then
        vowel = "a"
    elseif (lyric == "nyi") then
        vowel = "i"
    elseif (lyric == "nyu") then
        vowel = "u"
    elseif (lyric == "nye") then
        vowel = "e"
    elseif (lyric == "nyo") then
        vowel = "o"
    elseif (lyric == "hya") then
        vowel = "a"
    elseif (lyric == "hyi") then
        vowel = "i"
    elseif (lyric == "hyu") then
        vowel = "u"
    elseif (lyric == "hye") then
        vowel = "e"
    elseif (lyric == "hyo") then
        vowel = "o"
    elseif (lyric == "mya") then
        vowel = "a"
    elseif (lyric == "myi") then
        vowel = "i"
    elseif (lyric == "myu") then
        vowel = "u"
    elseif (lyric == "mye") then
        vowel = "e"
    elseif (lyric == "myo") then
        vowel = "o"
    elseif (lyric == "rya") then
        vowel = "a"
    elseif (lyric == "ryi") then
        vowel = "i"
    elseif (lyric == "ryu") then
        vowel = "u"
    elseif (lyric == "rye") then
        vowel = "e"
    elseif (lyric == "ryo") then
        vowel = "o"
    elseif (lyric == "wha") then
        vowel = "a"
    elseif (lyric == "whi") then
        vowel = "i"
    elseif (lyric == "whu") then
        vowel = "u"
    elseif (lyric == "whe") then
        vowel = "e"
    elseif (lyric == "who") then
        vowel = "o"
-- ローマ字処理(複数文字(濁音))
    elseif (lyric == "va") then
        vowel = "a"
    elseif (lyric == "vi") then
        vowel = "i"
    elseif (lyric == "vu") then
        vowel = "u"
    elseif (lyric == "ve") then
        vowel = "e"
    elseif (lyric == "vo") then
        vowel = "o"
    elseif (lyric == "gya") then
        vowel = "a"
    elseif (lyric == "gyi") then
        vowel = "i"
    elseif (lyric == "gyu") then
        vowel = "u"
    elseif (lyric == "gye") then
        vowel = "e"
    elseif (lyric == "gyo") then
        vowel = "o"
    elseif (lyric == "gwa") then
        vowel = "a"
    elseif (lyric == "gwi") then
        vowel = "i"
    elseif (lyric == "gwu") then
        vowel = "u"
    elseif (lyric == "gwe") then
        vowel = "e"
    elseif (lyric == "gwo") then
        vowel = "o"
    elseif (lyric == "ja") then
        vowel = "a"
    elseif (lyric == "ji") then
        vowel = "i"
    elseif (lyric == "ju") then
        vowel = "u"
    elseif (lyric == "je") then
        vowel = "e"
    elseif (lyric == "jo") then
        vowel = "o"
    elseif (lyric == "jya") then
        vowel = "a"
    elseif (lyric == "jyi") then
        vowel = "i"
    elseif (lyric == "jyu") then
        vowel = "u"
    elseif (lyric == "jye") then
        vowel = "e"
    elseif (lyric == "jyo") then
        vowel = "o"
    elseif (lyric == "zya") then
        vowel = "a"
    elseif (lyric == "zyi") then
        vowel = "i"
    elseif (lyric == "zyu") then
        vowel = "u"
    elseif (lyric == "zye") then
        vowel = "e"
    elseif (lyric == "zyo") then
        vowel = "o"
    elseif (lyric == "dya") then
        vowel = "a"
    elseif (lyric == "dyi") then
        vowel = "i"
    elseif (lyric == "dyu") then
        vowel = "u"
    elseif (lyric == "dye") then
        vowel = "e"
    elseif (lyric == "dyo") then
        vowel = "o"
    elseif (lyric == "dha") then
        vowel = "a"
    elseif (lyric == "dhi") then
        vowel = "i"
    elseif (lyric == "dhu") then
        vowel = "u"
    elseif (lyric == "dhe") then
        vowel = "e"
    elseif (lyric == "dho") then
        vowel = "o"
    elseif (lyric == "bya") then
        vowel = "a"
    elseif (lyric == "byi") then
        vowel = "i"
    elseif (lyric == "byu") then
        vowel = "u"
    elseif (lyric == "bye") then
        vowel = "e"
    elseif (lyric == "byo") then
        vowel = "o"
-- ローマ字処理(複数文字(半濁音))
    elseif (lyric == "pya") then
        vowel = "a"
    elseif (lyric == "pyi") then
        vowel = "i"
    elseif (lyric == "pyu") then
        vowel = "u"
    elseif (lyric == "pye") then
        vowel = "e"
    elseif (lyric == "pyo") then
        vowel = "o"
-- 例外
    else
        vowel = lyric
    end
    return vowel
end


-- 母音の音からキーフレーム用の値を求める関数
function vowelToKey(vowel)
    local key
    if (vowel == "a") then
        key = 1
    elseif (vowel == "i") then
        key = 2
    elseif (vowel == "u") then
        key = 3
    elseif (vowel == "e") then
        key = 4
    elseif (vowel == "o") then
        key = 5
    elseif (vowel == "n") then
        key = 6
    else
        key = 0
    end
    return key
end

